@@ -1,7 +1,7 @@
 #
 #   push-ws.rb -
 #   	$Release Version: 0.9.6$
-#   	$Revision: 25189 $
+#   	$Revision: 38515 $
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
@@ -9,13 +9,15 @@
 #
 #
 
-module IRB
+module IRB # :nodoc:
   class Context
 
+    # Size of the current WorkSpace stack
     def irb_level
       workspace_stack.size
     end
 
+    # WorkSpaces in the current stack
     def workspaces
       if defined? @workspaces
 	@workspaces
@@ -24,6 +26,11 @@ module IRB
       end
     end
 
+    # Creates a new workspace with the given object or binding, and appends it
+    # onto the current #workspaces stack.
+    #
+    # See IRB::Context#change_workspace and IRB::WorkSpace.new for more
+    # information.
     def push_workspace(*_main)
       if _main.empty?
 	if workspaces.empty?
@@ -43,6 +50,10 @@ module IRB
       end
     end
 
+    # Removes the last element from the current #workspaces stack and returns
+    # it, or +nil+ if the current workspace stack is empty.
+    #
+    # Also, see #push_workspace.
     def pop_workspace
       if workspaces.empty?
 	print "workspace stack empty\n"
