@@ -8,6 +8,7 @@
 #
 
 
+#--
 # == Developer Documentation (not for RDoc output)
 #
 # === Class tree
@@ -42,6 +43,8 @@
 #                         | all instances)|
 #                         +---------------+
 #
+#++
+#
 # == OptionParser
 #
 # === Introduction
@@ -108,7 +111,7 @@
 #       options.transfer_type = :auto
 #       options.verbose = false
 #
-#       opts = OptionParser.new do |opts|
+#       opt_parser = OptionParser.new do |opts|
 #         opts.banner = "Usage: example.rb [options]"
 #
 #         opts.separator ""
@@ -187,7 +190,7 @@
 #         end
 #       end
 #
-#       opts.parse!(args)
+#       opt_parser.parse!(args)
 #       options
 #     end  # parse()
 #
@@ -195,6 +198,7 @@
 #
 #   options = OptparseExample.parse(ARGV)
 #   pp options
+#   pp ARGV
 #
 # === Shell Completion
 #
@@ -204,12 +208,11 @@
 # === Further documentation
 #
 # The above examples should be enough to learn how to use this class.  If you
-# have any questions, email me (gsinclair@soyabean.com.au) and I will update
-# this document.
+# have any questions, file a ticket at http://bugs.ruby-lang.org.
 #
 class OptionParser
   # :stopdoc:
-  RCSID = %w$Id: optparse.rb 44935 2014-02-14 04:52:03Z usa $[1..-1].each {|s| s.freeze}.freeze
+  RCSID = %w$Id: optparse.rb 45050 2014-02-19 16:38:03Z nagachika $[1..-1].each {|s| s.freeze}.freeze
   Version = (RCSID[1].split('.').collect {|s| s.to_i}.extend(Comparable).freeze if RCSID[1])
   LastModified = (Time.gm(*RCSID[2, 2].join('-').scan(/\d+/).collect {|s| s.to_i}) if RCSID[2])
   Release = RCSID[2]
@@ -1336,6 +1339,7 @@ XXX
 
   #
   # Same as #order, but removes switches destructively.
+  # Non-option arguments remain in +argv+.
   #
   def order!(argv = default_argv, &nonopt)
     parse_in_order(argv, &nonopt)
@@ -1426,6 +1430,7 @@ XXX
 
   #
   # Same as #permute, but removes switches destructively.
+  # Non-option arguments remain in +argv+.
   #
   def permute!(argv = default_argv)
     nonopts = []
@@ -1445,6 +1450,7 @@ XXX
 
   #
   # Same as #parse, but removes switches destructively.
+  # Non-option arguments remain in +argv+.
   #
   def parse!(argv = default_argv)
     if ENV.include?('POSIXLY_CORRECT')
